@@ -49,22 +49,27 @@
 #include "intel_chipset.h"
 #include "igt_debugfs.h"
 
-/*
- * There hasn't been a release of libdrm containing these #define's yet, so
- * copy them here to allow compilation to succeed in the mean time.
- *
- * We can drop these #define's and just make i-g-t depend on the proper libdrm
- * version in the future.
- */
-#define DRM_CLIENT_CAP_UNIVERSAL_PLANES 2
-#define DRM_PLANE_TYPE_OVERLAY 0
-#define DRM_PLANE_TYPE_PRIMARY 1
-#define DRM_PLANE_TYPE_CURSOR  2
-
 /* list of connectors that need resetting on exit */
 #define MAX_CONNECTORS 32
 static char *forced_connectors[MAX_CONNECTORS + 1];
 
+static void update_edid_csum(unsigned char *edid)
+{
+	int i, sum = 0;
+	struct tm *tm;
+	time_t t;
+
+	/* year of manufacture */
+	t = time(NULL);
+	tm = localtime(&t);
+	edid[17] = tm->tm_year - 90;
+
+	/* calculate checksum */
+	for (i = 0; i < 127; i++) {
+		sum = sum + edid[i];
+	}
+	edid[127] = 256 - sum;
+}
 
 #define VFREQ 60
 #define CLOCK 148500
@@ -80,62 +85,8 @@ static char *forced_connectors[MAX_CONNECTORS + 1];
 #define HSIZE 52
 #define VSIZE 30
 
-#define GAMMA(x) (x * 100) - 100
-
-#define MANUFACTURER_ID(a, b, c) (a - '@') << 2 | (b - '@') >> 3, \
-				 (b - '@') << 5 | (c - '@')
-
-
-#define ab(x, y) (x & 0xff), (y & 0xff), ((x & 0xf00) >> 4) | ((y & 0xf00) >> 8)
-#define op(ho, hp, vo, vp) (ho & 0xff), (hp & 0xff), \
-			   ((vo & 0xf) << 4) | (vp & 0xf), \
-			   ((ho & 0x300) >> 2) | ((hp & 0x300) >> 4) \
-			   | ((vo & 0x30) >> 2) | (vp & 0x30 >> 4)
-
-static unsigned char base_edid[EDID_LENGTH] = {
-	0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, /* header */
-	MANUFACTURER_ID('I', 'G', 'T'),
-	/* product code, serial number, week and year of manufacture */
-	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-	0x01, 0x03, /* edid version (1.3) */
-	/* basic display parameters */
-	/* digital display, maximum horizontal image size, maximum vertical
-	 * image size, gamma, features: RGB 4:4:4, native pixel format and
-	 * refresh rate in descriptor 1 */
-	0x80, HSIZE, VSIZE, GAMMA(2.20), 0x02,
-	/* chromaticity coordinates */
-	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-	/* established timings: 640x480 60Hz, 800x600 60Hz, 1024x768 60Hz */
-	0x21, 0x08, 0x00,
-	/* standard timings */
-	0xd1, 0xc0, /* 1920x1080 60Hz */
-	0x81, 0xc0, /* 1280x720 60Hz */
-	0x61, 0x40, /* 1024x768 60Hz */
-	0x45, 0x40, /* 800x600 60Hz */
-	0x31, 0x40, /* 640x480 60Hz */
-	0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
-	/* descriptor 1 (preferred timing) */
-	(CLOCK / 10) & 0x00ff, ((CLOCK / 10) & 0xff00) >> 8,
-	ab(HACTIVE, HBLANK), ab(VACTIVE, VBLANK),
-	op(HOFFSET, HPULSE, VOFFSET, VPULSE),
-	ab(HSIZE * 10, VSIZE * 10),
-	0x00, 0x00, 0x00,
-	/* descriptor 2 (monitor range limits) */
-	0x00, 0x00, 0x00, 0xfd, 0x00,
-	VFREQ - 1, VFREQ + 1, /* minimum, maximum vertical field rate */
-	(CLOCK / (HACTIVE + HBLANK)) - 1, /* minimum horizontal line rate */
-	(CLOCK / (HACTIVE + HBLANK)) + 1, /* maximum horizontal line rate */
-	(CLOCK / 10000) + 1, /* maximum pixel clock rate */
-	0x00, 0x0a, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
-	/* descriptor 3 (name descriptor) */
-	0x00, 0x00, 0x00, 0xfc, 0x00,  'I',  'G',  'T', 0x0a,
-	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
-	/* descriptor 4 */
-	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-	/* extensions, checksum */
-	0x00, 0x00
-};
+#define EDID_NAME base_edid
+#include "igt_edid_template.h"
 
 /**
  * igt_kms_get_base_edid:
@@ -155,30 +106,57 @@ static unsigned char base_edid[EDID_LENGTH] = {
  */
 const unsigned char* igt_kms_get_base_edid(void)
 {
-	int i, sum = 0;
-	struct tm *tm;
-	time_t t;
-
-	/* year of manufacture */
-	t = time(NULL);
-	tm = localtime(&t);
-	base_edid[17] = tm->tm_year - 90;
-
-	/* calculate checksum */
-	for (i = 0; i < 127; i++) {
-		sum = sum + base_edid[i];
-	}
-	base_edid[127] = 256 - sum;
+	update_edid_csum(base_edid);
 
 	return base_edid;
 }
 
+#define VFREQ 60
+#define CLOCK 101000
+#define HACTIVE 1400
+#define HBLANK 160
+#define VACTIVE 1050
+#define VBLANK 30
+#define HOFFSET 48
+#define HPULSE 32
+#define VOFFSET 3
+#define VPULSE 4
+
+#define HSIZE 52
+#define VSIZE 30
+
+#define EDID_NAME alt_edid
+#include "igt_edid_template.h"
+
+/**
+ * igt_kms_get_alt_edid:
+ *
+ * Get an alternate edid block, which includes the following modes:
+ *
+ *  - 1400x1050 60Hz
+ *  - 1920x1080 60Hz
+ *  - 1280x720 60Hz
+ *  - 1024x768 60Hz
+ *  - 800x600 60Hz
+ *  - 640x480 60Hz
+ *
+ * This can be extended with further features using functions such as
+ * #kmstest_edid_add_3d.
+ *
+ * Returns: an alternate edid block
+ */
+const unsigned char* igt_kms_get_alt_edid(void)
+{
+	update_edid_csum(alt_edid);
+
+	return alt_edid;
+}
 
 /**
  * SECTION:igt_kms
  * @short_description: Kernel modesetting support library
- * @title: i-g-t kms
- * @include: igt_kms.h
+ * @title: KMS
+ * @include: igt.h
  *
  * This library provides support to enumerate and set modeset configurations.
  *
@@ -405,6 +383,7 @@ bool kmstest_force_connector(int drm_fd, drmModeConnector *connector,
 	char *path, **tmp;
 	const char *value;
 	int debugfs_fd, ret, len;
+	drmModeConnector *temp;
 	uint32_t devid;
 
 	devid = intel_get_drm_devid(drm_fd);
@@ -471,6 +450,11 @@ bool kmstest_force_connector(int drm_fd, drmModeConnector *connector,
 
 	igt_install_exit_handler(reset_connectors_at_exit);
 
+	/* To allow callers to always use GetConnectorCurrent we need to force a
+	 * redetection here. */
+	temp = drmModeGetConnector(drm_fd, connector->connector_id);
+	drmModeFreeConnector(temp);
+
 	igt_assert(ret != -1);
 	return (ret == -1) ? false : true;
 }
@@ -492,6 +476,7 @@ void kmstest_force_edid(int drm_fd, drmModeConnector *connector,
 {
 	char *path;
 	int debugfs_fd, ret;
+	drmModeConnector *temp;
 
 	igt_assert_neq(asprintf(&path, "%s-%d/edid_override", kmstest_connector_type_str(connector->connector_type), connector->connector_type_id),
 		       -1);
@@ -506,6 +491,11 @@ void kmstest_force_edid(int drm_fd, drmModeConnector *connector,
 		ret = write(debugfs_fd, edid, length);
 	close(debugfs_fd);
 
+	/* To allow callers to always use GetConnectorCurrent we need to force a
+	 * redetection here. */
+	temp = drmModeGetConnector(drm_fd, connector->connector_id);
+	drmModeFreeConnector(temp);
+
 	igt_assert(ret != -1);
 }
 
@@ -543,18 +533,20 @@ bool kmstest_get_connector_default_mode(int drm_fd, drmModeConnector *connector,
 }
 
 /**
- * kmstest_get_connector_config:
+ * _kmstest_connector_config:
  * @drm_fd: DRM fd
  * @connector_id: DRM connector id
  * @crtc_idx_mask: mask of allowed DRM CRTC indices
  * @config: structure filled with the possible configuration
+ * @probe: whether to fully re-probe mode list or not
  *
  * This tries to find a suitable configuration for the given connector and CRTC
  * constraint and fills it into @config.
  */
-bool kmstest_get_connector_config(int drm_fd, uint32_t connector_id,
-				  unsigned long crtc_idx_mask,
-				  struct kmstest_connector_config *config)
+static bool _kmstest_connector_config(int drm_fd, uint32_t connector_id,
+				      unsigned long crtc_idx_mask,
+				      struct kmstest_connector_config *config,
+				      bool probe)
 {
 	drmModeRes *resources;
 	drmModeConnector *connector;
@@ -568,7 +560,11 @@ bool kmstest_get_connector_config(int drm_fd, uint32_t connector_id,
 	}
 
 	/* First, find the connector & mode */
-	connector = drmModeGetConnector(drm_fd, connector_id);
+	if (probe)
+		connector = drmModeGetConnector(drm_fd, connector_id);
+	else
+		connector = drmModeGetConnectorCurrent(drm_fd, connector_id);
+
 	if (!connector)
 		goto err2;
 
@@ -643,6 +639,43 @@ err1:
 	return false;
 }
 
+/**
+ * kmstest_get_connector_config:
+ * @drm_fd: DRM fd
+ * @connector_id: DRM connector id
+ * @crtc_idx_mask: mask of allowed DRM CRTC indices
+ * @config: structure filled with the possible configuration
+ *
+ * This tries to find a suitable configuration for the given connector and CRTC
+ * constraint and fills it into @config.
+ */
+bool kmstest_get_connector_config(int drm_fd, uint32_t connector_id,
+				  unsigned long crtc_idx_mask,
+				  struct kmstest_connector_config *config)
+{
+	return _kmstest_connector_config(drm_fd, connector_id, crtc_idx_mask,
+					 config, 0);
+}
+
+/**
+ * kmstest_probe_connector_config:
+ * @drm_fd: DRM fd
+ * @connector_id: DRM connector id
+ * @crtc_idx_mask: mask of allowed DRM CRTC indices
+ * @config: structure filled with the possible configuration
+ *
+ * This tries to find a suitable configuration for the given connector and CRTC
+ * constraint and fills it into @config, fully probing the connector in the
+ * process.
+ */
+bool kmstest_probe_connector_config(int drm_fd, uint32_t connector_id,
+				    unsigned long crtc_idx_mask,
+				    struct kmstest_connector_config *config)
+{
+	return _kmstest_connector_config(drm_fd, connector_id, crtc_idx_mask,
+					 config, 1);
+}
+
 /**
  * kmstest_free_connector_config:
  * @config: connector configuration structure
@@ -836,7 +869,7 @@ void kmstest_unset_all_crtcs(int drm_fd, drmModeResPtr resources)
 	int i, rc;
 
 	for (i = 0; i < resources->count_crtcs; i++) {
-		rc = drmModeSetCrtc(drm_fd, resources->crtcs[i], -1, 0, 0, NULL,
+		rc = drmModeSetCrtc(drm_fd, resources->crtcs[i], 0, 0, 0, NULL,
 				    0, NULL);
 		igt_assert(rc == 0);
 	}
@@ -974,6 +1007,15 @@ static int get_drm_plane_type(int drm_fd, uint32_t plane_id)
 	return DRM_PLANE_TYPE_OVERLAY;
 }
 
+/**
+ * igt_display_init:
+ * @display: a pointer to an #igt_display_t structure
+ * @drm_fd: a drm file descriptor
+ *
+ * Initialize @display and allocate the various resources required. Use
+ * #igt_display_fini to release the resources when they are no longer required.
+ *
+ */
 void igt_display_init(igt_display_t *display, int drm_fd)
 {
 	drmModeRes *resources;
@@ -1172,6 +1214,13 @@ static void igt_output_fini(igt_output_t *output)
 	free(output->name);
 }
 
+/**
+ * igt_display_fini:
+ * @display: a pointer to an #igt_display_t structure
+ *
+ * Release any resources associated with @display. This does not free @display
+ * itself.
+ */
 void igt_display_fini(igt_display_t *display)
 {
 	int i;
@@ -1719,12 +1768,12 @@ drmModeModeInfo *igt_output_get_mode(igt_output_t *output)
 
 /**
  * igt_output_override_mode:
- * @output: Output of which the mode will be overriden
+ * @output: Output of which the mode will be overridden
  * @mode: New mode
  *
  * Overrides the output's mode with @mode, so that it is used instead of the
  * mode obtained with get connectors. Note that the mode is used without
- * checking if the output supports it, so this might lead to unexpect results.
+ * checking if the output supports it, so this might lead to unexpected results.
  */
 void igt_output_override_mode(igt_output_t *output, drmModeModeInfo *mode)
 {
@@ -1804,7 +1853,7 @@ void igt_plane_set_position(igt_plane_t *plane, int x, int y)
  * @h: height
  *
  * This function sets width and height for requested plane.
- * New size will be commited at plane commit time via
+ * New size will be committed at plane commit time via
  * drmModeSetPlane().
  */
 void igt_plane_set_size(igt_plane_t *plane, int w, int h)
@@ -1931,7 +1980,7 @@ void igt_crtc_set_background(igt_pipe_t *pipe, uint64_t background)
 {
 	igt_display_t *display = pipe->display;
 
-	LOG(display, "%s.%d: crtc_set_background(%lu)\n",
+	LOG(display, "%s.%d: crtc_set_background(%"PRIx64")\n",
 	    kmstest_pipe_name(pipe->pipe),
 	    pipe->pipe, background);
 
@@ -1969,13 +2018,13 @@ void igt_enable_connectors(void)
 	drmModeConnector *c;
 	int drm_fd;
 
-	drm_fd = drm_open_any();
+	drm_fd = drm_open_driver(DRIVER_INTEL);
 
 	res = drmModeGetResources(drm_fd);
 
 	for (int i = 0; i < res->count_connectors; i++) {
 
-		c = drmModeGetConnector(drm_fd, res->connectors[i]);
+		c = drmModeGetConnectorCurrent(drm_fd, res->connectors[i]);
 
 		/* don't attempt to force connectors that are already connected
 		 */
