@@ -29,50 +29,54 @@
  * Negative test cases for destroy contexts
   */
 
+#include "igt.h"
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
 
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
 
-struct local_drm_i915_context_destroy {
-	__u32 ctx_id;
-	__u32 pad;
-};
+IGT_TEST_DESCRIPTION("Negative test cases for destroy contexts.");
 
-#define CONTEXT_DESTROY_IOCTL DRM_IOWR(DRM_COMMAND_BASE + 0x2e, struct local_drm_i915_context_destroy)
+uint32_t ctx_id;
+int fd;
 
-igt_simple_main
+igt_main
 {
-	struct local_drm_i915_context_destroy destroy;
-	uint32_t ctx_id;
-	int ret, fd;
+	igt_fixture {
+		fd = drm_open_driver_render(DRIVER_INTEL);
 
-	igt_skip_on_simulation();
-
-	fd = drm_open_any_render();
-
-	ctx_id = gem_context_create(fd);
-
-	destroy.ctx_id = ctx_id;
-	/* Make sure a proper destroy works first */
-	ret = drmIoctl(fd, CONTEXT_DESTROY_IOCTL, &destroy);
-	igt_assert(ret == 0);
+		ctx_id = gem_context_create(fd);
+		/* Make sure a proper destroy works first */
+		gem_context_destroy(fd, ctx_id);
+	}
 
 	/* try double destroy */
-	ret = drmIoctl(fd, CONTEXT_DESTROY_IOCTL, &destroy);
-	igt_assert(ret != 0 && errno == ENOENT);
+	igt_subtest("double-destroy") {
+		ctx_id = gem_context_create(fd);
+		gem_context_destroy(fd, ctx_id);
+		igt_assert(__gem_context_destroy(fd, ctx_id) == -ENOENT);
+	}
 
-	/* destroy something random */
-	destroy.ctx_id = 2;
-	ret = drmIoctl(fd, CONTEXT_DESTROY_IOCTL, &destroy);
-	igt_assert(ret != 0 && errno == ENOENT);
+	igt_subtest("invalid-ctx")
+		igt_assert(__gem_context_destroy(fd, 2) == -ENOENT);
 
-	/* Try to destroy the default context */
-	destroy.ctx_id = 0;
-	ret = drmIoctl(fd, CONTEXT_DESTROY_IOCTL, &destroy);
-	igt_assert(ret != 0 && errno == ENOENT);
+	igt_subtest("invalid-default-ctx")
+		igt_assert(__gem_context_destroy(fd, 0) == -ENOENT);
 
-	close(fd);
+	igt_subtest("invalid-pad") {
+		struct drm_i915_gem_context_destroy destroy;
+
+		ctx_id = gem_context_create(fd);
+
+		memset(&destroy, 0, sizeof(destroy));
+		destroy.ctx_id = ctx_id;
+		destroy.pad = 1;
+
+		igt_assert(drmIoctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_DESTROY, &destroy) < 0 &&
+			   errno == EINVAL);
+		gem_context_destroy(fd, ctx_id);
+	}
+
+	igt_fixture
+		close(fd);
 }
