@@ -25,6 +25,7 @@
  *
  */
 
+#include "igt.h"
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -37,10 +38,6 @@
 #include <sys/ioctl.h>
 #include <sys/time.h>
 #include "drm.h"
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
-#include "intel_chipset.h"
-#include "intel_io.h"
 
 #define OBJECT_SIZE 16384
 
@@ -210,7 +207,7 @@ static void run(int object_size, bool dumb)
 	int fd, len, count;
 	int ring;
 
-	fd = drm_open_any();
+	fd = drm_open_driver(DRIVER_INTEL);
 	if (dumb)
 		handle = dumb_create(fd);
 	else
